(function () {
    if (window['WebViewJavascriptBridge']) {
        return;
    }

    if (typeof String.prototype.startsWith != 'function') {
        String.prototype.startsWith = function (prefix) {
            return this.slice(0, prefix.length) === prefix;
        };
    }

    var registerHandlers = {};
    var asyncHandles = {};

    function registerHandler(handlerName, handler) {
        registerHandlers[handlerName] = handler;
        window['_android_js_obj_']['registerEvent'](handlerName, handlerName)
    }

    function unRegisterHandler(handlerName) {
        delete registerHandlers[handlerName];
        window['_android_js_obj_']['unRegisterEvent'](handlerName)
    }

    function tryParseResultToJSONObject(result) {
        if (typeof (result) != 'string') return result;
        try {
            if (result.startsWith('{') || result.startsWith('[')) {
                result = JSON.parse(result);
            }
        } catch (ignore) { }
        return result;
    }

    function callHandler(handlerName, data, responseCallback) {
        if (typeof data == 'function') {
            responseCallback = data;
            data = void (0);
        }
        try {
            var result = null;
            if (data === void (0)) {
                result = window['_android_js_obj_'][handlerName]();
            } else {
                result = window['_android_js_obj_'][handlerName](JSON.stringify(data));
            }
            if(!responseCallback) return;
            result = tryParseResultToJSONObject(result);
            if (typeof(result) == 'string' && result.startsWith('async_callback_key_')) {
                asyncHandles[result] = responseCallback;
            } else {
                setTimeout(function () {
                    responseCallback(result);
                });
            }
        } catch (e) {
            console.log('handleName:' + handlerName);
            console.log('data:' + JSON.stringify(data));
            console.log(e.stack);
        }
    }

    function _callJsFuncWithResult(json) {
        var uniqueId = json.uniqueId;
        var fullFunc = json.jsFunc;
        var match = fullFunc.match(/^(.*?)\((.*)\)$/)
        var funcName = match[1]
        var data = match[2] || '{}'
        var func = registerHandlers[funcName]
        if (!func) {
            console.log(funcName + ' not register.');
            return;
        }
        try {
            if (data.startsWith('{')) {
                data = JSON.parse(data)
            } else if (data.match(/'.*'/)) {
                data = data.substr(1, data.length - 2);
            } else if (data.match(/".*"/)) {
                data = data.substr(1, data.length - 2);
            }
        } catch (ignore) {

        }
        function responseCallback(data) {
            window['_android_js_obj_']['__onResponseJSResult'](uniqueId, JSON.stringify(data));
        }
        func(data, responseCallback);
    }

    function _onAsyncFuncResult(json) {
        var key = json['key'];
        var data = json['data'];
        var func = asyncHandles[key]
        delete asyncHandles[key]
        if (!func) return;
        data = tryParseResultToJSONObject(data);
        func(data)
    }

    var WebViewJavascriptBridge = window['WebViewJavascriptBridge'] = {
        registerHandler: registerHandler,
        callHandler: callHandler,
        _callJsFuncWithResult: _callJsFuncWithResult,
        _onAsyncFuncResult: _onAsyncFuncResult
    };

    if (window['WVJBCallbacks']) {
        window['WVJBCallbacks'].forEach(function (cb) {
            cb(WebViewJavascriptBridge)
        })
    }
})();
